# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.urlresolvers import reverse_lazy
from django.db import models
from django.utils.translation import ugettext_lazy as _

class Actor(models.Model):
	first_name = models.CharField(max_length=100)
	last_name  = models.CharField(max_length=100)
	image      = models.ImageField(upload_to='actor/', blank=True, null=True)

	class Meta:
		ordering            = ['last_name', 'first_name']
		verbose_name        = _('Actor')
		verbose_name_plural = _('Actors')

	def __unicode__(self):
		return '%s, %s' % (self.last_name, self.first_name)

	def get_absolute_url(self):
		return reverse_lazy('store:actor_detail', args=[self.pk, ])

	def get_actions(self):
		return (
			( _('Edit'), 'update', 'info', 'pencil' ),
			( _('Delete'), 'delete', 'danger', 'trash' ),
		)

class Contact(models.Model):
	actor   = models.OneToOneField(Actor, related_name='contact_information') # The related_name is crucial
	phone   = models.CharField(max_length=30)
	email   = models.EmailField()
	address = models.TextField(blank=True)

	def __unicode__(self):
		return '%s' % self.actor

class Movie(models.Model):
	name        = models.CharField(max_length=250)
	year        = models.PositiveIntegerField()
	description = models.TextField()
	image       = models.ImageField(upload_to='movies/',blank=True, null=True)

	class Meta:
		ordering            = ['-year', 'name']
		verbose_name        = _('Movie')
		verbose_name_plural = _('Movies')

	def __unicode__(self):
		return '%s' % self.name

	def get_absolute_url(self):
		return reverse_lazy('store:movie_detail', args=[self.pk, ])

class Cast(models.Model):
	movie       = models.ForeignKey(Movie)
	actor       = models.ForeignKey(Actor)
	name        = models.CharField(max_length=250)
	description = models.TextField(blank=True)
	image       = models.ImageField(upload_to='casts/', blank=True, null=True)

	class Meta:
		ordering            = ['name',]
		verbose_name        = _('Cast')
		verbose_name_plural = _('Casts')
	
	def __unicode__(self):
		return '%s' % self.name

	def get_absolute_url(self):
		return reverse_lazy('store:cast_detail', args=[self.pk, ])


# -*- coding: utf-8 -*-
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType

def add_view_permissions(sender, **kwargs):
	for content_type in ContentType.objects.all():
		codename = "view_%s" % content_type.model

		if not Permission.objects.filter(content_type=content_type, codename=codename):
			Permission.objects.create(content_type=content_type,
				codename=codename,
				name='Can view %s' % content_type.name)
			print 'Added view permission for %s' % content_type.name


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _

class AccountConfig(AppConfig):
	name         = 'account'
	verbose_name = _('Account')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from importlib import import_module

# Default settings
BOILERPLATE_DEFAULTS = {
	'site_name': '',
	'messages': {
		'change_password_current': _('You can\'t use your old password. Type a new password.'),
		'change_password_new': _('Passwords don\'t match.'),
		'login_credentials': _('Wrong username or password. Please try again.'),
		'login_inactive': _('Your account has been deactivated. Please contact the site adminitrator.'),
		'recover_unknown': _('We couldn\'t find your username or email.'),
		'registration_email': _('Email missmatch.'),
		'registration_password': _('Password missmatch.'),
		'registration_username_already': _('Username already taken.'),
		'registration_email_already': _('Another account has that email.'),
		'recover_sent': _('An email has been sent to %(email)s. Please check its inbox to continue reseting password.'),
		'recover_subject': _('Recover your account information'),
	}
}

# Start with a copy of default settings
BOILERPLATE = BOILERPLATE_DEFAULTS.copy()

# Override with user settings from settings.py
BOILERPLATE.update(getattr(settings, 'BOILERPLATE', {}))

def get_boilerplate_setting(setting, default=None):
	"""
	Read a setting
	"""
	return BOILERPLATE.get(setting, default)

def messages(**kwargs):
	"""
	Return the dict with the messages
	"""
	return get_boilerplate_setting('messages')

def site_name(**kwargs):
	"""
	Return the the full site_name
	"""
	return get_boilerplate_setting('site_name')

def recover_url_name(**kwargs):
	"""
	Return the the full recover_url_name
	"""
	return get_boilerplate_setting('recover_url_name')


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

try:
	from StringIO import StringIO
except ImportError:
	from io import StringIO

from django.core.files.base import ContentFile
from django.core.files.storage import default_storage as storage

from PIL import Image

class ModelImageThumbs(object):
	IMAGESIZES = None

	def save(self, *args, **kwargs):
		response = super(ModelImageThumbs, self).save(*args, **kwargs)

		if self.image:
			filename  = str(self.image.name)
			imgFile   = Image.open(storage.open(filename))

			if imgFile.mode not in ('L', 'RGB'):
				imgFile = imgFile.convert('RGB')

			for field_name, size in self.IMAGESIZES.iteritems():
				field   = getattr(self, field_name)

				if not field:
					working = imgFile.copy()
					working = working.resize(size, Image.ANTIALIAS)
					fp      = StringIO()
					working.save(fp, 'JPEG', quality=95)
					cf      = ContentFile( fp.getvalue() )
					field.save(name=self.image.name, content=cf, save=True)
		else:
			for field_name, size in self.IMAGESIZES.iteritems():
				field = getattr(self, field_name)
				field.delete(save=True)

		return response


# -*- coding: utf-8 -*-
from django import forms
from django.contrib.auth import authenticate
from django.contrib.auth.models import User
from django.contrib.auth.password_validation import validate_password
from django.core.exceptions import ValidationError
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from .boilerplate import messages

class ChangePasswordForm(forms.ModelForm):
	password         = forms.CharField(label=_('Current Password'), max_length=30, widget=forms.PasswordInput())
	password_new     = forms.CharField(label=_('New password'), max_length=30, widget=forms.PasswordInput())
	password_confirm = forms.CharField(label=_('Confirm new password'), max_length=30, widget=forms.PasswordInput())
	
	class Meta:
		fields = ('password', 'password_new', 'password_confirm')
		model  = User

	def clean(self):
		cleaned_data     = super(ChangePasswordForm, self).clean()
		password         = cleaned_data['password']
		password_new     = cleaned_data['password_new']
		password_confirm = cleaned_data['password_confirm']

		if password_new != password_confirm:
			self.add_error('password_new', messages().get('change_password_new') )
			self.add_error('password_confirm', messages().get('change_password_new'))
		
		elif password == password_new:
			self.add_error('password', '')
			self.add_error('password_new', '')
			self.add_error('password_confirm', '')

			raise ValidationError( messages().get('change_password_current') )

		password_error = validate_password(password_new)

		if password_error is not None:
			return password_error

	def save(self, commit=True):
		user     = super(ChangePasswordForm, self).save(commit=False)
		password = self.cleaned_data['password_new']
		
		if commit:
			user.set_password(password)
			user.save()

		return user

class LoginForm(forms.Form):
	username = forms.CharField( label=_('Username') )
	password = forms.CharField( label=_('Password'), widget=forms.PasswordInput() )

	def clean(self):
		cleaned_data = super(LoginForm, self).clean()
		username     = cleaned_data['username']
		password     = cleaned_data['password']
		user         = authenticate(username=username, password=password)

		if user is None:
			self.add_error('username', '')
			self.add_error('password', '')
			
			raise ValidationError( messages().get('login_credentials') )
		elif not user.is_active:
			self.add_error('username', '')
			self.add_error('password', '')
			
			raise ValidationError( messages().get('login_inactive') )

class RecoverAccountForm(forms.Form):
	username = forms.CharField( label='Username or Email', max_length=150 )

	def clean_username(self):
		data = self.cleaned_data['username']

		if not User.objects.filter( Q(username=data) | Q(email=data) ).exists():
			raise ValidationError( messages().get('recover_unknown') )

		return data

class RecoverAccountConfirmForm(forms.Form):
	password_new     = forms.CharField(label=_('New password'), max_length=30, widget=forms.PasswordInput())
	password_confirm = forms.CharField(label=_('Confirm new password'), max_length=30, widget=forms.PasswordInput())

	def clean(self):
		cleaned_data     = super(RecoverAccountConfirmForm, self).clean()
		password_new     = cleaned_data['password_new']
		password_confirm = cleaned_data['password_confirm']

		if password_new != password_confirm:
			self.add_error('password_new', messages().get('change_password_new'))
			self.add_error('password_confirm', messages().get('change_password_new'))

		password_error = validate_password(password_new)

		if password_error is not None:
			return password_error

class RegistrationForm(forms.ModelForm):
	username   = forms.CharField( label=_('Username') )
	email      = forms.EmailField( label=_('Email') )
	email2     = forms.EmailField( label=_('Confirm Email') )
	password   = forms.CharField( label=_('Password'), widget=forms.PasswordInput() )
	password2  = forms.CharField( label=_('Confirm Password'), widget=forms.PasswordInput() )
	first_name = forms.CharField( label=_('Name') )
	last_name  = forms.CharField( label=_('Last Name') )

	class Meta:
		model  = User
		fields = ['username', 'email', 'email2', 'password', 'password2', 'first_name', 'last_name']

	def clean(self):
		cleaned_data = super(RegistrationForm, self).clean()
		email     = cleaned_data.get('email')
		email2    = cleaned_data.get('email2')
		password  = cleaned_data.get('password')
		password2 = cleaned_data.get('password2')

		if password != password2:
			self.add_error('password',  messages().get('registration_password'))
			self.add_error('password2', messages().get('registration_password'))

		if email != email2:
			self.add_error('email', messages().get('registration_email'))
			self.add_error('email2', messages().get('registration_email'))

	def clean_username(self):
		data = self.cleaned_data['username']

		if User.objects.filter(username=data).exists():
			raise forms.ValidationError(messages().get('registration_username_already'))

		return data

	def clean_email(self):
		data = self.cleaned_data['email']

		if User.objects.filter(email=data).exists():
			raise forms.ValidationError(messages().get('registration_email_already'))

		return data

	def save(self, commit=True):
		user = super(RegistrationForm, self).save(commit=False)
		user.set_password(self.cleaned_data['password'])

		if commit:
			user.save()
			
		return user


# -*- coding: utf-8 -*-
from django.contrib import messages
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.models import User
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.tokens import default_token_generator
from django.db.models import Q
from django.shortcuts import redirect
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode, urlsafe_base64_decode
from django.utils.translation import ugettext_lazy as _
from django.views.generic import View
from django.views.generic.edit import CreateView, FormView, UpdateView

from . import forms
from boilerplate import messages as boilerplate_messages, recover_url_name, site_name
from mail import SendEmail
from mixins import NoLoginRequiredMixin

class ChangePasswordView(LoginRequiredMixin, UpdateView):
	form_class    = forms.ChangePasswordForm
	model         = User
	success_url   = '/'

	def get_object(self):
		return self.request.user

	def form_valid(self, form):
		user = authenticate(username=self.request.user.username, password=form.cleaned_data['password'])

		if user is None:
			form.add_error('username', '')
			form.add_error('password', '')

			raise ValidationError(_('Password is incorrect. Please verify your password.'))

		return super(ChangePasswordView, self).form_valid(form)

class LoginView(NoLoginRequiredMixin, FormView):
	form_class = forms.LoginForm

	def form_valid(self, form):
		response = super(LoginView, self).form_valid(form)

		username = form.cleaned_data['username']
		password = form.cleaned_data['password']

		user = authenticate(username=username, password=password)

		if user is not None and user.is_active:
			login(self.request, user)

		return response

	def get_success_url(self):
		if self.request.GET.get('next', None):
			return self.request.GET.get('next')
		else:
			return '/'

class LogoutView(LoginRequiredMixin, View):
	def get(self, request, **kwargs):
		logout(request)

		if request.GET.get('next', None):
			return redirect(request.GET.get('next'))
		else:
			return redirect('/')

class RecoverAccountView(NoLoginRequiredMixin, FormView):
	form_class  = forms.RecoverAccountForm
	success_url = '/'
	recover_url = None
 
	def form_valid(self, form):
		username = form.cleaned_data['username']
		user     = User.objects.get( Q(username=username) | Q(email=username) )
		object   = {
			'email': user.email,
			'domain': self.request.META['HTTP_HOST'],
			'url_name': self.recover_url,
			'uid': urlsafe_base64_encode(force_bytes(user.pk)),
			'user': user,
			'token': default_token_generator.make_token(user),
			'protocol': 'http',
		}
		messages.success(self.request, boilerplate_messages().get('recover_sent') % dict(email=user.email))

		mail = SendEmail(to=user.email, template_name_suffix='recover-account', subject=boilerplate_messages().get('recover_subject'), is_html=True)
		mail.set_from_name('No Reply')
		mail.add_context_data('object', object)
		mail.send()
		
		return super(RecoverAccountView, self).form_valid(form)

class RecoverAccountConfirmView(NoLoginRequiredMixin, FormView):
	form_class    = forms.RecoverAccountConfirmForm
	success_url   = '/'

	def post(self, request, uidb64=None, token=None, *arg, **kwargs):
		form = self.get_form()
		assert uidb64 is not None and token is not None  # checked by URLconf
		
		try:
			uid  = urlsafe_base64_decode(uidb64)
			user = User.objects.get(pk=uid)
		except (TypeError, ValueError, OverflowError, User.DoesNotExist):
			user = None

		if user is not None and default_token_generator.check_token(user, token):
			if form.is_valid():
				user.set_password(form.cleaned_data['password_new'])
				user.save()
				messages.success(request, _('Password has been reset successfully.'))
				return self.form_valid(form)
			else:
				messages.error(request, _('Password reset has been unsuccessful.'))
				return self.form_invalid(form)
		else:
			messages.error(request, _('The reset password link is no longer valid. Request a new one.'))
			return self.form_invalid(form)

class RegistrationView(NoLoginRequiredMixin, CreateView):
	form_class    = forms.RegistrationForm
	model         = User
	success_url   = '/'


# -*- coding: utf-8 -*-
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404, redirect
from django.utils import six
from django.utils.translation import ugettext_lazy as _


class NoLoginRequiredMixin(object):
	"""
	Mixin for any class view classes that required the current user
	if not authenticated, redirects the user to the home page or any
	URL with the *next* parameter.

	**Example**
	::
		class Register(NoLoginRequiredMixin, FormView):
			form_class = forms.RegisterForm
	"""
	def get(self, request, **kwargs):
		if request.user.is_authenticated():
			raise PermissionDenied

		return super(NoLoginRequiredMixin, self).get(request, **kwargs)

class ListActionsMixin(object):
	"""
	Mixin for :class:`~django.views.generic.list.ListView` classes that
	adds to the "context" a variable with a list of actions.

	**Example**
	::
		class ModelList(ListActionsMixin, ListView):
			actions = (
				_('Add'), 'create', 'primary', 'plus'),
				_('Export'), 'export', 'success', 'export'),
			)
			model = Model
	"""
	def get_action_list(self):
		"""
		Return the list of actions to show in the "context".

		Override this function to personalize the list of actions.

		For example: based on the user permissions.
		"""
		return self.action_list

	def get_context_data(self, **kwargs):
		if not 'actions' in kwargs:
			kwargs['actions'] = self.get_action_list()

		return super(ListActionsMixin, self).get_context_data(**kwargs)

class CreateModelMixin(object):
	def form_valid(self, form):
		setattr(form.instance, self.field_user, self.request.user)

		return super(CreateModelMixin, self).form_valid(form)

class CRUDMessageMixin(object):
	message_action  = None
	success_message = _('%(model)s: "%(name)s" has been %(action)s successfully.')
	
	def get_success_message(self):
		"""
		Default success message mixin
		"""
		return self.success_message % dict(
			model  = self.object.__class__._meta.verbose_name.title(),
			name   = self.object,
			action = self.message_action,
		)

	def get_success_url(self):
		"""
		If the `get_sucess_url` if overwrite the message wont appear
		"""
		success_message = self.get_success_message()
		
		if success_message:
			messages.success(self.request, success_message)
		
		return super(CRUDMessageMixin, self).get_success_url()

class CreateMessageMixin(CRUDMessageMixin):
	"""
	Mixin for :class:`~django.views.generic.edit.CreateView` classes that
	adds a success message after the action is completed successfully.

	**Example**
	::
		class ModelSendEmail(CreateMessageMixin, CreateView):
			model = Model
	"""
	message_action = 'created'

class UpdateMessageMixin(CRUDMessageMixin):
	"""
	Mixin for :class:`~django.views.generic.edit.UpdateView` classes that
	adds a success message after the action is completed successfully.

	**Example**
	::
		class ModelSendEmail(UpdateMessageMixin, UpdateView):
			model = Model
	"""
	message_action = 'updated'

class DeleteMessageMixin(CRUDMessageMixin):
	"""
	Mixin for :class:`~django.views.generic.edit.DeleteView` classes that
	adds a success message after the action is completed successfully.

	**Example**
	::
		class ModelSendEmail(DeleteMessageMixin, DeleteView):
			model = Model
	"""
	message_action = 'deleted'

class ExtraFormsAndFormsetsMixin(object):
	"""
	Mixin for :class:`~django.views.generic.edit.CreateView` or
	:class:`~django.views.generic.edit.UpdateView` classes that
	adds extra forms and formsets to any of thoose views.

	**Example**
	::
		class UserUpdate(ExtraFormsAndFormsetsMixin, UpdateView):
			model = User
			extra_form_list = (
				('profile', 'user', forms.ProfileForm),
				('credit_card', 'user', forms.ProfileForm),
			)
			formset_list = (
				(form.AddressFormSet),
				(form.PhoneFormSet),
			)
	"""
	extra_form_list = None
	formset_list    = None

	def get_extra_form_list(self):
		"""
		Returns a list of extra forms with the lookup_field,
		relation_field and form_class to use in this view.
		"""
		return self.extra_form_list

	def get_formset_list(self):
		"""
		Returns a list of formsets with the formset class to use in this view.
		"""
		return self.formset_list

	def get_extra_forms(self, form_list=None):
		"""
		Returns a list of each extra forms to be used in this view.
		"""
		output = list()
		
		if form_list is None:
			form_list = self.get_extra_form_list()

		if form_list:
			for lookup_field, relation_field, extra_form in form_list:
				output.append(
					(relation_field, extra_form(**self.get_extra_form_kwargs(lookup_field)))
				)

		return output

	def get_formsets(self, formset_list=None):
		"""
		Returns a list of formsets to be used in this view.
		"""
		output = list()
		
		if formset_list is None:
			formset_list = self.get_formset_list()

		if formset_list:
			for formset in formset_list:
				output.append(formset(**self.get_formset_kwargs()))

		return output

	def get_extra_form_kwargs(self, lookup_field):
		"""
		Returns the keyword arguments for instantiating each extra form.
		"""
		kwargs = {
			'prefix': lookup_field,
		}

		if self.request.method in ('POST', 'PUT'):
			kwargs.update({
				'data': self.request.POST,
				'files': self.request.FILES,
			})

		if hasattr(self, 'object'):
			try:
				kwargs.update({'instance': getattr(self.object, lookup_field)})
			except:
				pass

		return kwargs


	def get_formset_kwargs(self):
		"""
		Returns the keyword arguments for instantiating each formset.
		"""
		kwargs = {}

		if self.request.method in ('POST', 'PUT'):
			kwargs.update({
				'data': self.request.POST,
				'files': self.request.FILES,
			})

		if hasattr(self, 'object'):
			kwargs.update({'instance': self.object})

		return kwargs

	def post(self, request, *args, **kwargs):
		"""
		Handles POST requests, instantiating a form instance with the passed
		POST variables and then checked for validity.
		"""
		try:
			self.object = self.get_object()
		except:
			self.object = None

		form        = self.get_form()
		extra_forms = self.get_extra_forms()
		formsets    = self.get_formsets()

		if not form.is_valid():
			return self.form_invalid(form, extra_forms, formsets)

		for relation_field, extra_form in extra_forms:
			if not extra_form.is_valid():
				return self.form_invalid(form, extra_forms, formsets)
		
		for formset in formsets:
			if not formset.is_valid():
				return self.form_invalid(form, extra_forms, formsets)

		return self.form_valid(form, extra_forms, formsets)

	def form_valid(self, form, extra_forms, formsets):
		"""
		If the form is valid, redirect to the supplied URL.
		"""
		self.object = form.save()

		for relation_field, extra_form in extra_forms:
			setattr(extra_form.instance, relation_field, self.object)
			extra_form.save()

		for formset in formsets:
			formset.instance = self.object
			formset.save()
		
		return HttpResponseRedirect(self.get_success_url())

	def form_invalid(self, form, extra_forms, formsets):
		"""
		If the form or the extra forms are invalid, re-render the context data with the
		data-filled form and errors.
		"""
		return self.render_to_response(self.get_context_data(form=form, extra_forms=extra_forms, formsets=formsets))

	def get_context_data(self, **kwargs):
		"""
		Insert the extra forms and formsets into the context dict.
		"""
		if 'extra_forms' not in kwargs:
			kwargs['extra_forms'] = list()
			
			for relation_field, extra_form in self.get_extra_forms():
				kwargs['extra_forms'].append(extra_form)
		else:
			extra_forms = list()
			
			for relation_field, extra_form in kwargs['extra_forms']:
				extra_forms.append(extra_form)
			
			kwargs['extra_forms'] = extra_forms

		if 'formsets' not in kwargs:
			kwargs['formsets'] = self.get_formsets()
		
		return super(ExtraFormsAndFormsetsMixin, self).get_context_data(**kwargs)

class ParentMixin(object):
	"""
	Mixin for :class:`~django.views.generic.edit.CreateView` or
	:class:`~django.views.generic.edit.UpdateView` classes that
	adds a success message after the action is completed successfully.

	**Example**
	::
		class ProfileUpdate(ParentMixin, UpdateView):
			model               = Profile
			parent_model        = User
			parent_lookup_arg   = 'pk_parent'
			parent_lookup_field = 'pk'
	"""
	parent_model          = None
	parent_lookup_arg     = 'pk_parent'
	parent_lookup_field   = 'pk'

	def get_parent_model(self):
		return self.parent_model

	def get_parent_lookup_arg(self):
		return self.parent_lookup_arg

	def get_parent_lookup_field(self):
		return self.parent_lookup_field

	def get_parent_kwargs(self):
		kwargs = {
			self.get_parent_lookup_field(): self.kwargs[self.get_parent_lookup_arg()],
		}

		return kwargs

	def get_parent(self):
		return get_object_or_404(self.get_parent_model(), **self.get_parent_kwargs())

	def get_context_data(self, **kwargs):
		if 'parent' not in kwargs:
			kwargs['parent'] = self.get_parent()

		return super(ParentMixin, self).get_context_data(**kwargs)

class ParentCreateMixin(ParentMixin):
	parent_set_field = None

	def get_parent_set_field(self):
		return self.parent_set_field

	def form_valid(self, form):
		setattr(form.instance, self.get_parent_set_field(), self.get_parent())

		return super(ParentCreateMixin, self).form_valid(form)

class ParentCreateExtraMixin(ParentCreateMixin):
	def form_valid(self, form, extra_forms, formsets):
		setattr(form.instance, self.get_parent_set_field(), self.get_parent())

		return super(ParentCreateExtraMixin, self).form_valid(form, extra_forms, formsets)

class ParentSingleObjectMixin(ParentMixin):
	parent_relation_field = None

	def get_parent_relation_field(self):
		return self.parent_relation_field

	def get_queryset(self):
		qs = super(ParentSingleObjectMixin, self).get_queryset()

		kwargs = {
			self.get_parent_relation_field(): self.get_parent(),
		}

		return qs.filter(**kwargs)


# -*- coding: utf-8 -*-
import unicodedata
from django import template
from django.contrib.admin.utils import NestedObjects
from django.core.urlresolvers import reverse
from django.db import DEFAULT_DB_ALIAS
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _

register = template.Library()

@register.simple_tag
def url_replace(request, field, value):
	dict_        = request.GET.copy()
	dict_[field] = value
	return dict_.urlencode()

@register.filter
def get_deleted_objects(object):
	"""
	List the related objects before delete an object
	"""
	collector = NestedObjects(using=DEFAULT_DB_ALIAS)
	collector.collect([object])
	
	return collector.nested()

"""

Form filters

"""
@register.filter
def form_model_name(value):
	"""
	Return the model verbose name of a form model
	"""
	try:
		return value._meta.model._meta.verbose_name
	except:
		return value.__class__.__name__

@register.filter
def form_model_name_plural(value):
	"""
	Return the model verbose name plural of a form model
	"""
	return value._meta.model._meta.verbose_name_plural

@register.filter
def form_model_url(value):
	"""
	Return the model list url name of a form model
	"""
	return value._meta.model._meta.app_label + ':' + value._meta.model.__name__.lower() + '_list'

@register.filter
def form_app_name(value):
	"""
	Return the app name of a form model
	"""
	return value._meta.model._meta.app_config.verbose_name

@register.filter
def form_app_url(value):
	"""
	Return the app home url of a form model
	"""
	return value._meta.model._meta.app_label.lower() + ':home'

@register.filter
def form_prefix(value):
	"""
	Return the form name as a prefix
	"""
	return slugify(value.__class__.__name__.lower())

@register.filter
def formset_model_name(value):
	"""
	Return the model verbose name of a formset
	"""
	return value.model._meta.verbose_name

@register.filter
def formset_model_name_plural(value):
	"""
	Return the model verbose name plural of a formset
	"""
	return value.model._meta.verbose_name_plural

"""

Model filters

"""
@register.simple_tag
def model_action(value, action):
	"""
	**Tag name**
	::
		model_action
	
	Return the full url name of an action
	
	**Usage**
	::
		{% model_action object action %}
	
	**Example**
	::
		{% model_action object 'update' %}
	"""
	name =  value._meta.model._meta.app_label + ':' + value._meta.model.__name__.lower() + "_" + action
	return reverse(name, args=(value.pk,))

@register.simple_tag
def model_child_action(value, parent, action):
	"""
	**Tag name**
	::
		model_child_action
	
	Return the full url name of an action
	
	**Usage**
	::
		{% model_child_action object parent action %}
	
	**Example**
	::
		{% model_child_action object parent 'update' %}
	"""
	name =  value._meta.model._meta.app_label + ':' + parent._meta.model.__name__.lower() + "_" + value._meta.model.__name__.lower() + "_" + action
	return reverse(name, args=(parent.pk, value.pk,))

@register.filter
def model_name(value):
	"""
	Return the model verbose name of an object
	"""
	return value._meta.verbose_name

@register.filter
def model_name_plural(value):
	"""
	Return the model verbose name plural of an object
	"""
	return value._meta.verbose_name_plural

@register.filter
def model_app_name(value):
	"""
	Return the app verbose name of an object
	"""
	return value._meta.app_config.verbose_name

@register.filter
def model_app_url(value):
	"""
	Return the app home url of an object
	"""
	return value._meta.app_label.lower().replace(" ", "_") + ":home"

@register.filter
def model_url(value):
	"""
	Return the model list url name of a model
	"""
	return value._meta.model._meta.app_label + ':' + value._meta.model.__name__.lower() + '_list'

"""

Queryset filters

"""
@register.filter
def queryset_app_name(value):
	"""
	Return the app verbose name of a queryset
	"""
	return value.model._meta.app_config.verbose_name

@register.filter
def queryset_app_url(value):
	"""
	Return the app home url name of a queryset
	"""
	return value.model._meta.app_label.lower() + ":home"

@register.filter
def queryset_model_name_plural(value):
	"""
	Return the app verbose name plural of a queryset
	"""
	return value.model._meta.verbose_name_plural

@register.filter
def queryset_model_url(value):
	"""
	Return the model list url name of a queryset
	"""
	return value.model.__name__.lower() + '_list'

@register.simple_tag
def queryset_action(value, action):
	"""
	**Tag name**
	::
		queryset_action
	
	Return the full url name of an action

	**Parameters**:
			:queryset: A valid queryset of objects

			:action: A valid action name. Ex. namespace:model_action
	
	**Usage**
	::
		{% queryset_action queryset action %}
	
	**Example**
	::
		{% queryset_action queryset 'add' %}
	"""
	name = value.model._meta.app_label + ':' +  value.model.__name__.lower() + "_" + action
	return reverse(name)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template

class SendEmail(object):
	"""
	Send an emails or several emails easily.

	**Example**
	::
		email = SendEmail(to='user@example.com', template_name_suffix='recover-account', subject='Recover your account', is_html=True)
		email.set_from_email('no-reply@example.com')
		email.set_from_name('No Reply')
		email.add_context_data('protocol', 'https')
		email.add_context_data('domain', 'example.com')
		email.add_context_data('uid', uid)
		email.add_context_data('token', token)
		email.add_context_data('object', user)
		email.add_context_data('site_name', 'Boilerplate - Make it easy')
		email.send()
	"""

	context_data   = {}
	files         = list()
	from_name     = None
	from_email    = None
	template_name = None

	def __init__(self, to, template_name_suffix, subject, is_html=False):
		if isinstance(to, list):
			self.to = to
		else:
			self.to = list()
			self.to.append(to)

		self.template_name_suffix = template_name_suffix
		self.subject              = subject
		self.is_html              = is_html

	def add_file(self, file):
		self.files.append(file)

	def add_context_data(self, key, value):
		"""
		Add a key-value to the context data of the email

		**Parameters**:
			:key: A valid key name
			:value: Can be an object or any kind of value
		"""
		self.context_data.update({
			str(key): value
		})

	def set_from_email(self, from_email=None):
		"""
		Set the email sender

		**Parameters**:
			:from_email: String, a valid email
		"""
		if from_email:
			self.from_email = str(from_email)
		else:
			self.from_email = settings.SERVER_EMAIL

	def set_from_name(self, from_name=None):
		"""
		Set the name sender

		**Parameters**:
			:from_name: String, a valid name
		"""
		if from_name:
			self.from_name = str(from_name)
		else:
			self.from_name = settings.SERVER_EMAIL

	def set_template_name_suffix(self, template_name_suffix=None):
		"""
		Set the email template name suffix

		**Parameters**:
			:template_name_suffix: String: the name of the template without the extension
		"""
		if template_name_suffix:
			self.template_name_suffix = str(template_name_suffix)

	def set_template_name(self, template_name=None):
		"""
		Set the email template name

		**Parameters**:
			:template_name: String: the name of the template without the extension
		"""
		if template_name:
			self.template_name = template_name
		else:
			if not self.template_name_suffix:
				self.set_template_name_suffix()

			self.template_name = 'mail/' + self.template_name_suffix

	def get_from_email(self):
		if not self.from_email:
			self.set_from_email()
		
		return self.from_email

	def get_from_name(self):
		if not self.from_name:
			self.set_from_name()
		
		return self.from_name

	def get_template_name_suffix(self):
		if not self.template_name_suffix:
			self.set_template_name_suffix()

		return self.template_name_suffix

	def get_template_name(self):
		if not self.template_name:
			self.set_template_name()		

		return self.template_name

	def get_context_data(self, **kwargs):
		self.context_data.update({
			'email': self
		})

		return self.context_data

	def send(self, fail_silently=True):
		plain_template = get_template(self.get_template_name() + '.txt')
		plain_content  = plain_template.render(self.get_context_data())

		if self.is_html:
			html_template  = get_template(self.get_template_name() + '.html')
			html_content   = html_template.render(self.get_context_data())

		msg = EmailMultiAlternatives(self.subject, plain_content, '%s <%s>' % (self.get_from_name(), self.get_from_email()), self.to)

		if self.is_html:
			msg.attach_alternative(html_content, 'text/html')

		if self.files:
			for name, file, content_type in files:
				msg.attach(name, file.read(), content_type)

		return msg.send(fail_silently=fail_silently)


# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-17 21:02
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Actor',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=100)),
                ('last_name', models.CharField(max_length=100)),
                ('image', models.ImageField(blank=True, null=True, upload_to='actor/')),
            ],
            options={
                'ordering': ['last_name', 'first_name'],
                'verbose_name': 'Actor',
                'verbose_name_plural': 'Actors',
            },
        ),
        migrations.CreateModel(
            name='Cast',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=250)),
                ('description', models.TextField(blank=True)),
                ('image', models.ImageField(blank=True, null=True, upload_to='casts/')),
                ('actor', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='store.Actor')),
            ],
            options={
                'ordering': ['name'],
                'verbose_name': 'Cast',
                'verbose_name_plural': 'Casts',
            },
        ),
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('phone', models.CharField(max_length=30)),
                ('email', models.EmailField(max_length=254)),
                ('address', models.TextField(blank=True)),
                ('actor', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='contact_information', to='store.Actor')),
            ],
        ),
        migrations.CreateModel(
            name='Movie',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=250)),
                ('year', models.PositiveIntegerField()),
                ('description', models.TextField()),
                ('image', models.ImageField(blank=True, null=True, upload_to='movies/')),
            ],
            options={
                'ordering': ['-year', 'name'],
                'verbose_name': 'Movie',
                'verbose_name_plural': 'Movies',
            },
        ),
        migrations.AddField(
            model_name='cast',
            name='movie',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='store.Movie'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import boilerplate

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

version = boilerplate.__version__

if sys.argv[-1] == 'publish':
    os.system('cd docs && make html')
    os.system('python setup.py sdist upload')
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

setup(
    name='django-boilerplate',
    version=version,
    description="""What Django is missing""",
    long_description=readme + '\n\n' + history,
    author='Irving Kcam',
    author_email='me@ikcam.com',
    url='https://github.com/cubope/django-boilerplate',
    packages=[
        'boilerplate',
    ],
    include_package_data=True,
    extras_require={
        'Pillow': ['pillow'],
    },
    license="Apache License 2.0",
    zip_safe=False,
    keywords=['django', 'boilerplate'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',
        'Environment :: Web Environment',
    ],
)